# _*_ coding:utf-8 _*_
import base64
import hashlib
import time
import requests
import logging

from wiz_utils.string_utils import StringUtils

import hmac

from wiz_dingtalk.message import Message


class DingTalkBot(object):
    base_url = "https://oapi.dingtalk.com/robot/send"

    def __init__(self, token: str, secret: str = None):
        self._secret = secret
        self._token = token

    @property
    def secret(self):
        return self._secret

    @property
    def token(self):
        return self._token

    def send(self, message: Message = None, json_message: str = None):
        ts = int(time.time() * 1000)
        params = {"access_token": self.token, "timestamp": ts}
        if StringUtils.is_not_blank(self.secret):
            string_to_sign = "%s\n%s" % (ts, self.secret)
            sign = base64.b64encode(hmac.new(self.secret.encode("utf-8"), string_to_sign.encode("utf-8"),
                                             digestmod=hashlib.sha256).digest())
            params['sign'] = sign
        try:
            request_body = message.to_json() if message is not None else json_message
            if StringUtils.is_blank(request_body):
                raise AttributeError("request body is null")
            response = requests.post(url=self.base_url, params=params,
                                     headers={"Content-Type": "application/json; charset=utf-8"}, data=request_body)
            print(response.content)
        except Exception as e:
            logging.error("send message error", e)



import abc
import json


class Message(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def to_json(self):
        pass


class TextMessage(Message):
    def __init__(self):
        self._title = None
        self._content = None

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, title: str):
        self._title = title

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value: str):
        self._content = value

    def to_json(self):
        if self.title is None or self.content is None:
            raise AttributeError("title or content is null")
        return json.dumps({"msgtype": "text", "text": {"title": self.title, "content": self._content}})


class MarkdownMessage(Message):
    def __init__(self):
        self._title = None
        self._content = None

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, title: str):
        self._title = title

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value: str):
        self._content = value

    def to_json(self):
        if self.title is None or self.content is None:
            raise AttributeError("title or content is null")
        return json.dumps({"msgtype": "markdown", "markdown": {"title": self.title, "text": self._content}})



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='wiz-dingtalk',

    version='1.0.2',

    description='wiz-dingtalk',

    long_description=long_description,

    long_description_content_type='text/markdown',

    url='https://chnyangjie.github.io/',

    author='chnyangjie',

    author_email='chnyangjie@gmail.com',

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],

    keywords='python dingtalk client',
    package_dir={'wiz_dingtalk': 'src/wiz_dingtalk'},
    packages=find_packages(where='src'),
    python_requires='>=3.6, <4',
    install_requires=['requests', 'wiz-utils'],
    py_modules=['wiz_dingtalk'],
    project_urls={
        'Bug Reports': 'https://github.com/chnyangjie/wiz_dingtalk/issues',
        'Say Thanks!': 'https://github.com/chnyangjie/wiz_dingtalk/issues',
        'Source': 'https://github.com/chnyangjie/wiz_dingtalk',
    },
)
